var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};

// src/HttpStatus.ts
var HttpStatusEnum = /* @__PURE__ */ ((HttpStatusEnum2) => {
  HttpStatusEnum2[HttpStatusEnum2["OK"] = 200] = "OK";
  HttpStatusEnum2[HttpStatusEnum2["CREATED"] = 201] = "CREATED";
  HttpStatusEnum2[HttpStatusEnum2["ACCEPTED"] = 202] = "ACCEPTED";
  HttpStatusEnum2[HttpStatusEnum2["NO_CONTENT"] = 204] = "NO_CONTENT";
  HttpStatusEnum2[HttpStatusEnum2["BAD_REQUEST"] = 400] = "BAD_REQUEST";
  HttpStatusEnum2[HttpStatusEnum2["UNAUTHORIZED"] = 401] = "UNAUTHORIZED";
  HttpStatusEnum2[HttpStatusEnum2["FORBIDDEN"] = 403] = "FORBIDDEN";
  HttpStatusEnum2[HttpStatusEnum2["NOT_FOUND"] = 404] = "NOT_FOUND";
  HttpStatusEnum2[HttpStatusEnum2["INTERNAL_SERVER_ERROR"] = 500] = "INTERNAL_SERVER_ERROR";
  HttpStatusEnum2[HttpStatusEnum2["SERVICE_UNAVAILABLE"] = 503] = "SERVICE_UNAVAILABLE";
  return HttpStatusEnum2;
})(HttpStatusEnum || {});
var HttpStatusEnumText = {
  [200 /* OK */]: "OK",
  [201 /* CREATED */]: "Created",
  [202 /* ACCEPTED */]: "Accepted",
  [204 /* NO_CONTENT */]: "No Content",
  [400 /* BAD_REQUEST */]: "Bad Request",
  [401 /* UNAUTHORIZED */]: "Unauthorized",
  [403 /* FORBIDDEN */]: "Forbidden",
  [404 /* NOT_FOUND */]: "Not Found",
  [500 /* INTERNAL_SERVER_ERROR */]: "Internal Server Error",
  [503 /* SERVICE_UNAVAILABLE */]: "Service Unavailable"
};
var HttpStatus = class {
  /**
   * Retrieves the textual description of a given HTTP status code.
   * @param status The HTTP status code.
   * @returns The textual description of the status code, or "Unknown Status" if not found.
   */
  static getStatusText(status) {
    return HttpStatusEnumText[status] || "Unknown Status";
  }
  /**
   * Validates and converts a numeric HTTP status code to the corresponding `HttpStatusEnum` value.
   * @param statusCode The numeric HTTP status code.
   * @returns The corresponding `HttpStatusEnum` value if valid, or `undefined` otherwise.
   */
  static fromCode(statusCode) {
    return Object.values(HttpStatusEnum).includes(statusCode) ? statusCode : void 0;
  }
  /**
   * Retrieves both the numeric HTTP status code and its textual description.
   * @param statusCode The numeric HTTP status code.
   * @returns An object containing the status code and its description, or `undefined` if the code is invalid.
   */
  static getStatusWithText(statusCode) {
    const status = this.fromCode(statusCode);
    if (status) {
      return { status, text: this.getStatusText(status) };
    }
    return void 0;
  }
};
HttpStatus.OK = 200 /* OK */;
HttpStatus.CREATED = 201 /* CREATED */;
HttpStatus.ACCEPTED = 202 /* ACCEPTED */;
HttpStatus.NO_CONTENT = 204 /* NO_CONTENT */;
HttpStatus.BAD_REQUEST = 400 /* BAD_REQUEST */;
HttpStatus.UNAUTHORIZED = 401 /* UNAUTHORIZED */;
HttpStatus.FORBIDDEN = 403 /* FORBIDDEN */;
HttpStatus.NOT_FOUND = 404 /* NOT_FOUND */;
HttpStatus.INTERNAL_SERVER_ERROR = 500 /* INTERNAL_SERVER_ERROR */;
HttpStatus.SERVICE_UNAVAILABLE = 503 /* SERVICE_UNAVAILABLE */;

// src/errors.ts
var ControllerError = class extends Error {
  // The associated HTTP status code for the error.
  /**
   * Constructor for the `ControllerError` class.
   * @param key A unique key identifying the error.
   * @param message An optional detailed message for the error. If not provided, a default message is generated using the key.
   * @param httpStatus The HTTP status code for the error (default is 400 - Bad Request).
   */
  constructor(key, message, httpStatus = 400) {
    const defaultMessage = message || `An error occurred with key: ${key}`;
    super(defaultMessage);
    this.key = key;
    this.message = defaultMessage;
    this.httpStatus = httpStatus;
    this.name = this.constructor.name;
    if (Error.captureStackTrace) {
      Error.captureStackTrace(this, this.constructor);
    }
  }
};

// src/index.ts
import { NextResponse as NextResponse2 } from "next/server";

// src/controllerBuilders.ts
import { ZodError } from "zod";
import { NextResponse } from "next/server";
function buildResponse({
  data,
  response,
  contentType = "application/json"
}) {
  const headers = new Headers(response.headers);
  const statusEnum = HttpStatus.fromCode(response.httpStatus);
  const statusText = statusEnum !== void 0 ? HttpStatus.getStatusText(statusEnum) : null;
  headers.set("Content-Type", contentType);
  const body = contentType === "application/json" ? JSON.stringify(data) : data;
  return new NextResponse(body, __spreadProps(__spreadValues({
    status: response.httpStatus
  }, statusText ? { statusText } : {}), {
    headers
  }));
}
function buildError({
  key,
  message,
  response
}) {
  return buildResponse({
    data: {
      error: __spreadValues({ key }, message && { message })
    },
    response
  });
}
function handleError(error) {
  if (error instanceof ZodError) {
    return buildError({
      key: "VALIDATION_ERROR",
      message: error.errors.map((e) => e.message).join(", ") || "",
      response: { httpStatus: HttpStatus.BAD_REQUEST }
    });
  }
  if (error instanceof ControllerError) {
    return buildError({
      key: error.key,
      message: error.message,
      response: { httpStatus: error.httpStatus }
    });
  }
  return buildError({
    key: "UNEXPECTED_ERROR",
    message: error instanceof Error ? error.message : "Unknown error occurred",
    response: { httpStatus: HttpStatus.BAD_REQUEST }
  });
}

// src/index.ts
function getAuthorizationToken(request) {
  const bearerPrefix = "Bearer ";
  const authorizationHeader = request.headers.get("Authorization") || "";
  if (!authorizationHeader.startsWith(bearerPrefix)) return "";
  return authorizationHeader.slice(bearerPrefix.length);
}
var CustomHeaders = class extends Headers {
  constructor(request) {
    super(request.headers);
    this.request = request;
  }
  getBearerToken() {
    return getAuthorizationToken(this.request);
  }
  /**
   * Parses and returns cookies as a key-value object.
   * @returns An object representing the cookies.
   */
  getCookies() {
    const cookieHeader = this.get("cookie") || "";
    return Object.fromEntries(
      cookieHeader.split(";").map((cookie) => {
        const [key, value] = cookie.split("=").map((part) => part.trim());
        return [key, decodeURIComponent(value)];
      })
    );
  }
};
function controller(handler) {
  const handleRequest = (request, context) => __async(this, null, function* () {
    const params = context.params ? yield context.params : {};
    const searchParams = Object.fromEntries(
      request.nextUrl.searchParams.entries()
    );
    const requestBody = request.method !== "GET" ? yield request.json().catch(() => void 0) : void 0;
    const ctx = {
      request,
      context: {
        params,
        searchParams
      },
      body: requestBody,
      headers: new CustomHeaders(request)
    };
    try {
      const result = yield handler(ctx);
      if (result instanceof NextResponse2) {
        return result;
      }
      return buildResponse({
        data: result.data,
        response: { httpStatus: result.response.httpStatus }
      });
    } catch (error) {
      return handleError(error);
    }
  });
  handleRequest.authorize = function(authorizationCheck) {
    return (request, context) => __async(this, null, function* () {
      try {
        const token = getAuthorizationToken(request);
        const isAuthorized = yield authorizationCheck(token);
        if (!isAuthorized) {
          throw new ControllerError(
            "UNAUTHORIZED",
            "Unauthorized resource access",
            HttpStatus.UNAUTHORIZED
          );
        }
        return handleRequest(request, context);
      } catch (error) {
        return handleError(error);
      }
    });
  };
  return handleRequest;
}
export {
  ControllerError,
  HttpStatus,
  buildResponse,
  controller
};
//# sourceMappingURL=index.mjs.map